using System;
using System.Collections.Generic;
using System.Text;
using System.Net;
using System.Net.Sockets;

namespace MyN5980AExample
{
    class N5980A
    {
        /// m_Client holds the TcpClient that is used to establish 
        /// the remote connection.
        TcpClient m_Client = null;
        /// m_Stream holds the NetworkStream that provides 
        /// the communication link.
        NetworkStream m_Stream = null;


        /// The main entry point for the application.
        [STAThread]
        static void Main()
        {
            N5980A instrument = new N5980A();
            // Connect to the N5980A software running on the same
            // PC as this program and that is using port number 5025
            instrument.Connect("localhost", 5025);
            // set the data rate to OC-3
            instrument.Send(":FREQ OC3");
            // read the data rate
            string response = instrument.Send(":FREQ?");
            Console.WriteLine("Frequency: " + response);
            Console.WriteLine("Press Enter to continue");
            Console.ReadLine();
            // Close the socket => disconnect from the instrument
            instrument.Disconnect();
            // From here on, it is no longer possible to communicate with the
            // instrument.
        }
        

        /// Connect to the instrument with a given hostname and 
        /// port number. Both values can be retrieved by going to
        /// the menu bar in the N5980A user interface:
        /// Connection-->Info
        /// Use the IP Address (or hostname of the PC that runs
        /// the N5980A User Interface) and use the port number
        /// of the User Interface software instance to
        /// communicate with.
        public void Connect(string host, int port)
        {
            try
            {
                // Resolve address of the given hostname
                IPHostEntry ipHostEntry = Dns.GetHostEntry(host);
                IPAddress ipAddress = ipHostEntry.AddressList[0];
                // create TcpClient and connect it to the instrument
                m_Client = new TcpClient();
                m_Client.Connect(new IPEndPoint(ipAddress, port));
                m_Stream = m_Client.GetStream();
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
                Console.WriteLine("Press Enter to continue");
                Console.ReadLine();
            }
        } 
        

        /// Close the connection with the N5980A software.
        public void Disconnect()
        { 
        if (m_Client != null) 
            m_Client.Close(); 
        }
        

        /// Send a N5980A SCPI command to the Windows socket and 
        /// return the response (if it was a query).
        /// String contains the SCPI command to be written to 
        /// the Windows socket</param>
        /// A response of the instrument or an empty string
        /// if there is no response is returned.
        public string Send(string scpi_string)
        {
            string response = ""; 
            int i;
            /// Bufferin holds the input buffer when reading from the 
            /// Windows socket.
            byte[] bufferin = new byte[64 * 1024];
            // Write the string to the network socket 
            byte[] bufferout = new byte[scpi_string.Length + 1];
            for (i = 0; i < scpi_string.Length; i++)
                bufferout[i] = (byte)scpi_string[i];
            bufferout[scpi_string.Length] = (byte)'\n';
            m_Stream.Write(bufferout, 0, bufferout.Length);
            // If the string containes a '?', then it is a query and
            // the response needs to be read from the instrument
            if (scpi_string.IndexOf('?') != -1)
            {
                // Read resonse from network stream
                int numRead = m_Stream.Read(bufferin, 0, bufferin.Length);
                // if the response contained any bytes, then convert them
                // to a valid string
                if (numRead > 0)
                {
                    StringBuilder sb = new StringBuilder();
                    for (i = 0; i < numRead; i++)
                        sb.Append((char)bufferin[i]);
                    response = sb.ToString();
                }
            }
            return response;
        }
    }
}
